@@ -1,24 +1,12 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* weather-iwin.c - US National Weather Service IWIN forecast source
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, see
- * <http://www.gnu.org/licenses/>.
+ * SPDX-FileCopyrightText: The GWeather authors
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
+
+#include "gweather-private.h"
 
 #include <ctype.h>
 #include <stdlib.h>
@@ -26,8 +14,6 @@
 
 #include <libxml/parser.h>
 
-#include "gweather-private.h"
-
 static gboolean
 hasAttr (xmlNode *node, const char *attr_name, const char *attr_value)
 {
@@ -42,7 +28,7 @@ hasAttr (xmlNode *node, const char *attr_name, const char *attr_value)
     if (!attr)
         return res;
 
-    res = g_str_equal ((const char *)attr, attr_value);
+    res = g_str_equal ((const char *) attr, attr_value);
 
     xmlFree (attr);
 
@@ -50,19 +36,19 @@ hasAttr (xmlNode *node, const char *attr_name, const char *attr_value)
 }
 
 static GSList *
-parseForecastXml (const char *buff, GWeatherInfo *master_info)
+parseForecastXml (const char *buff, GWeatherInfo *original_info)
 {
     GSList *res = NULL;
     xmlDocPtr doc;
     xmlNode *root, *node;
 
-    g_return_val_if_fail (master_info != NULL, NULL);
+    g_return_val_if_fail (original_info != NULL, NULL);
 
     if (!buff || !*buff)
         return NULL;
 
-    #define XC (const xmlChar *)
-    #define isElem(_node,_name) g_str_equal ((const char *)_node->name, _name)
+#define XC                   (const xmlChar *)
+#define isElem(_node, _name) g_str_equal ((const char *) _node->name, _name)
 
     doc = xmlParseMemory (buff, strlen (buff));
     if (!doc)
@@ -77,7 +63,7 @@ parseForecastXml (const char *buff, GWeatherInfo *master_info)
         if (isElem (node, "data")) {
             xmlNode *n;
             char *time_layout = NULL;
-            time_t update_times[7] = {0};
+            gint64 update_times[7] = { 0 };
 
             for (n = node->children; n; n = n->next) {
                 if (!n->name)
@@ -93,17 +79,19 @@ parseForecastXml (const char *buff, GWeatherInfo *master_info)
                                 xmlChar *val = xmlNodeGetContent (c);
 
                                 if (val) {
-                                    time_layout = g_strdup ((const char *)val);
+                                    time_layout = g_strdup ((const char *) val);
                                     xmlFree (val);
                                 }
                             } else if (c->name && isElem (c, "start-valid-time")) {
                                 xmlChar *val = xmlNodeGetContent (c);
 
                                 if (val) {
-                                    GTimeVal tv;
+                                    GDateTime *dt;
 
-                                    if (g_time_val_from_iso8601 ((const char *)val, &tv)) {
-                                        update_times[count] = tv.tv_sec;
+                                    dt = g_date_time_new_from_iso8601 ((const char *) val, NULL);
+                                    if (dt != NULL) {
+                                        update_times[count] = g_date_time_to_unix (dt);
+                                        g_date_time_unref (dt);
                                     } else {
                                         update_times[count] = 0;
                                     }
@@ -132,9 +120,9 @@ parseForecastXml (const char *buff, GWeatherInfo *master_info)
                     if (!res) {
                         int i;
 
-                        for (i = 0; i < 7;  i++) {
-                            GWeatherInfo *nfo = _gweather_info_new_clone (master_info);
-			    nfo->priv->current_time = nfo->priv->update = update_times[i];
+                        for (i = 0; i < 7; i++) {
+                            GWeatherInfo *nfo = _gweather_info_new_clone (original_info);
+                            nfo->current_time = nfo->update = update_times[i];
 
                             if (nfo)
                                 res = g_slist_append (res, nfo);
@@ -152,28 +140,27 @@ parseForecastXml (const char *buff, GWeatherInfo *master_info)
 
                             for (c = p->children; c && at; c = c->next) {
                                 if (isElem (c, "value")) {
-                                    GWeatherInfo *nfo = (GWeatherInfo *)at->data;
-				    GWeatherInfoPrivate *priv = nfo->priv;
+                                    GWeatherInfo *nfo = (GWeatherInfo *) at->data;
                                     xmlChar *val = xmlNodeGetContent (c);
 
                                     /* can pass some values as <value xsi:nil="true"/> */
                                     if (!val || !*val) {
                                         if (is_max)
-                                            priv->temp_max = priv->temp_min;
+                                            nfo->temp_max = nfo->temp_min;
                                         else
-                                            priv->temp_min = priv->temp_max;
+                                            nfo->temp_min = nfo->temp_max;
                                     } else {
                                         if (is_max)
-                                            priv->temp_max = atof ((const char *)val);
+                                            nfo->temp_max = atof ((const char *) val);
                                         else
-                                            priv->temp_min = atof ((const char *)val);
+                                            nfo->temp_min = atof ((const char *) val);
                                     }
 
                                     if (val)
                                         xmlFree (val);
 
-                                    priv->tempMinMaxValid = priv->tempMinMaxValid || (priv->temp_max > -999.0 && priv->temp_min > -999.0);
-                                    priv->valid = priv->tempMinMaxValid;
+                                    nfo->tempMinMaxValid = nfo->tempMinMaxValid || (nfo->temp_max > -999.0 && nfo->temp_min > -999.0);
+                                    nfo->valid = nfo->tempMinMaxValid;
 
                                     at = at->next;
                                 }
@@ -185,29 +172,29 @@ parseForecastXml (const char *buff, GWeatherInfo *master_info)
                             for (c = p->children; c && at; c = c->next) {
                                 if (c->name && isElem (c, "weather-conditions")) {
                                     GWeatherInfo *nfo = at->data;
-				    GWeatherInfoPrivate *priv = nfo->priv;
                                     xmlChar *val = xmlGetProp (c, XC "weather-summary");
 
                                     if (val && nfo) {
                                         /* Checking from top to bottom, if 'value' contains 'name', then that win,
                                            thus put longer (more precise) values to the top. */
                                         unsigned int i;
-                                        struct _ph_list {
+                                        struct _ph_list
+                                        {
                                             const char *name;
                                             GWeatherConditionPhenomenon ph;
                                         } ph_list[] = {
-                                            { "Ice Crystals", GWEATHER_PHENOMENON_ICE_CRYSTALS } ,
-                                            { "Volcanic Ash", GWEATHER_PHENOMENON_VOLCANIC_ASH } ,
-                                            { "Blowing Sand", GWEATHER_PHENOMENON_SANDSTORM } ,
-                                            { "Blowing Dust", GWEATHER_PHENOMENON_DUSTSTORM } ,
-                                            { "Blowing Snow", GWEATHER_PHENOMENON_FUNNEL_CLOUD } ,
-                                            { "Drizzle", GWEATHER_PHENOMENON_DRIZZLE } ,
-                                            { "Rain", GWEATHER_PHENOMENON_RAIN } ,
-                                            { "Snow", GWEATHER_PHENOMENON_SNOW } ,
-                                            { "Fog", GWEATHER_PHENOMENON_FOG } ,
-                                            { "Smoke", GWEATHER_PHENOMENON_SMOKE } ,
-                                            { "Sand", GWEATHER_PHENOMENON_SAND } ,
-                                            { "Haze", GWEATHER_PHENOMENON_HAZE } ,
+                                            { "Ice Crystals", GWEATHER_PHENOMENON_ICE_CRYSTALS },
+                                            { "Volcanic Ash", GWEATHER_PHENOMENON_VOLCANIC_ASH },
+                                            { "Blowing Sand", GWEATHER_PHENOMENON_SANDSTORM },
+                                            { "Blowing Dust", GWEATHER_PHENOMENON_DUSTSTORM },
+                                            { "Blowing Snow", GWEATHER_PHENOMENON_FUNNEL_CLOUD },
+                                            { "Drizzle", GWEATHER_PHENOMENON_DRIZZLE },
+                                            { "Rain", GWEATHER_PHENOMENON_RAIN },
+                                            { "Snow", GWEATHER_PHENOMENON_SNOW },
+                                            { "Fog", GWEATHER_PHENOMENON_FOG },
+                                            { "Smoke", GWEATHER_PHENOMENON_SMOKE },
+                                            { "Sand", GWEATHER_PHENOMENON_SAND },
+                                            { "Haze", GWEATHER_PHENOMENON_HAZE },
                                             { "Dust", GWEATHER_PHENOMENON_DUST } /*,
                                             { "", GWEATHER_PHENOMENON_SNOW_GRAINS } ,
                                             { "", GWEATHER_PHENOMENON_ICE_PELLETS } ,
@@ -220,35 +207,36 @@ parseForecastXml (const char *buff, GWeatherInfo *master_info)
                                             { "", GWEATHER_PHENOMENON_TORNADO } ,
                                             { "", GWEATHER_PHENOMENON_DUST_WHIRLS } */
                                         };
-                                        struct _sky_list {
+                                        struct _sky_list
+                                        {
                                             const char *name;
                                             GWeatherSky sky;
                                         } sky_list[] = {
-                                            { "Mostly Sunny", GWEATHER_SKY_BROKEN } ,
-                                            { "Mostly Clear", GWEATHER_SKY_BROKEN } ,
-                                            { "Partly Cloudy", GWEATHER_SKY_SCATTERED } ,
-                                            { "Mostly Cloudy", GWEATHER_SKY_FEW } ,
-                                            { "Sunny", GWEATHER_SKY_CLEAR } ,
-                                            { "Clear", GWEATHER_SKY_CLEAR } ,
-                                            { "Cloudy", GWEATHER_SKY_OVERCAST } ,
-                                            { "Clouds", GWEATHER_SKY_SCATTERED } ,
-                                            { "Rain", GWEATHER_SKY_SCATTERED } ,
+                                            { "Mostly Sunny", GWEATHER_SKY_BROKEN },
+                                            { "Mostly Clear", GWEATHER_SKY_BROKEN },
+                                            { "Partly Cloudy", GWEATHER_SKY_SCATTERED },
+                                            { "Mostly Cloudy", GWEATHER_SKY_FEW },
+                                            { "Sunny", GWEATHER_SKY_CLEAR },
+                                            { "Clear", GWEATHER_SKY_CLEAR },
+                                            { "Cloudy", GWEATHER_SKY_OVERCAST },
+                                            { "Clouds", GWEATHER_SKY_SCATTERED },
+                                            { "Rain", GWEATHER_SKY_SCATTERED },
                                             { "Snow", GWEATHER_SKY_SCATTERED }
                                         };
 
-                                        priv->valid = TRUE;
+                                        nfo->valid = TRUE;
 
                                         for (i = 0; i < G_N_ELEMENTS (ph_list); i++) {
-                                            if (strstr ((const char *)val, ph_list [i].name)) {
-						priv->cond.significant = TRUE;
-                                                priv->cond.phenomenon = ph_list [i].ph;
+                                            if (strstr ((const char *) val, ph_list[i].name)) {
+                                                nfo->cond.significant = TRUE;
+                                                nfo->cond.phenomenon = ph_list[i].ph;
                                                 break;
                                             }
                                         }
 
                                         for (i = 0; i < G_N_ELEMENTS (sky_list); i++) {
-                                            if (strstr ((const char *)val, sky_list [i].name)) {
-                                                priv->sky = sky_list [i].sky;
+                                            if (strstr ((const char *) val, sky_list[i].name)) {
+                                                nfo->sky = sky_list[i].sky;
                                                 break;
                                             }
                                         }
@@ -271,9 +259,8 @@ parseForecastXml (const char *buff, GWeatherInfo *master_info)
                            They should be all valid or all invalid. */
                         for (r = res; r; r = r->next) {
                             GWeatherInfo *nfo = r->data;
-			    GWeatherInfoPrivate *priv = nfo->priv;
 
-                            if (!nfo || !priv->valid) {
+                            if (!nfo || !nfo->valid) {
                                 if (r->data)
                                     g_object_unref (r->data);
 
@@ -281,8 +268,8 @@ parseForecastXml (const char *buff, GWeatherInfo *master_info)
                             } else {
                                 have_any = TRUE;
 
-                                if (priv->tempMinMaxValid)
-                                    priv->temp = (priv->temp_min + priv->temp_max) / 2.0;
+                                if (nfo->tempMinMaxValid)
+                                    nfo->temp = (nfo->temp_min + nfo->temp_max) / 2.0;
                             }
                         }
 
@@ -305,40 +292,53 @@ parseForecastXml (const char *buff, GWeatherInfo *master_info)
     }
     xmlFreeDoc (doc);
 
-    #undef XC
-    #undef isElem
+#undef XC
+#undef isElem
 
     return res;
 }
 
 static void
-iwin_finish (SoupSession *session, SoupMessage *msg, gpointer data)
+iwin_finish (GObject *source, GAsyncResult *result, gpointer data)
 {
     GWeatherInfo *info;
-    GWeatherInfoPrivate *priv;
     WeatherLocation *loc;
+    SoupSession *session = SOUP_SESSION (source);
+    SoupMessage *msg = soup_session_get_async_result_message (session, result);
+    GBytes *body;
+    GError *error = NULL;
+    const char *content;
 
-    if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
+    body = soup_session_send_and_read_finish (session, result, &error);
+
+    if (!body) {
         /* forecast data is not really interesting anyway ;) */
-	if (msg->status_code == SOUP_STATUS_CANCELLED) {
-	    g_debug ("Failed to get IWIN forecast data: %d %s\n",
-		     msg->status_code, msg->reason_phrase);
-	    return;
-	}
-	g_warning ("Failed to get IWIN forecast data: %d %s\n",
-		   msg->status_code, msg->reason_phrase);
+        if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+            g_debug ("Failed to get IWIN forecast data: %s", error->message);
+            return;
+        }
+        g_warning ("Failed to get IWIN forecast data: %s", error->message);
+        g_clear_error (&error);
+        _gweather_info_request_done (data, msg);
+        return;
+    } else if (!SOUP_STATUS_IS_SUCCESSFUL (soup_message_get_status (msg))) {
+        g_bytes_unref (body);
+        g_warning ("Failed to get IWIN forecast data: [status: %d]: %s",
+                   soup_message_get_status (msg),
+                   soup_message_get_reason_phrase (msg));
         _gweather_info_request_done (data, msg);
         return;
     }
 
     info = data;
-    priv = info->priv;
-    loc = &priv->location;
+    loc = &info->location;
+    content = g_bytes_get_data (body, NULL);
 
     g_debug ("iwin data for %s", loc->zone);
-    g_debug ("%s", msg->response_body->data);
+    g_debug ("%s", content);
 
-    priv->forecast_list = parseForecastXml (msg->response_body->data, info);
+    info->forecast_list = parseForecastXml (content, info);
+    g_bytes_unref (body);
 
     _gweather_info_request_done (info, msg);
 }
@@ -347,7 +347,6 @@ iwin_finish (SoupSession *session, SoupMessage *msg, gpointer data)
 gboolean
 iwin_start_open (GWeatherInfo *info)
 {
-    GWeatherInfoPrivate *priv;
     gchar *url;
     WeatherLocation *loc;
     SoupMessage *msg;
@@ -358,20 +357,20 @@ iwin_start_open (GWeatherInfo *info)
 
     g_assert (info != NULL);
 
-    priv = info->priv;
-    loc = &priv->location;
+    loc = &info->location;
 
     /* No zone (or -) means no weather information from national offices.
        We don't actually use zone, but it's a good indicator of a US location.
        (@ and : prefixes were used in the past for Australia and UK) */
     if (!loc->zone || loc->zone[0] == '-' || loc->zone[0] == '@' || loc->zone[0] == ':') {
         g_debug ("iwin_start_open, ignoring location %s because zone '%s' has no weather info",
-                 loc->name, loc->zone ? loc->zone : "(empty)");
+                 loc->name,
+                 loc->zone ? loc->zone : "(empty)");
         return FALSE;
     }
 
     if (!loc->latlon_valid)
-	return FALSE;
+        return FALSE;
 
     /* see the description here: http://www.weather.gov/forecasts/xml/ */
     now = time (NULL);
@@ -380,11 +379,15 @@ iwin_start_open (GWeatherInfo *info)
     latstr = _radians_to_degrees_str (loc->latitude);
     lonstr = _radians_to_degrees_str (loc->longitude);
     url = g_strdup_printf ("https://www.weather.gov/forecasts/xml/sample_products/browser_interface/ndfdBrowserClientByDay.php?&lat=%s&lon=%s&format=24+hourly&startDate=%04d-%02d-%02d&numDays=7",
-			   latstr, lonstr, 1900 + tm.tm_year, 1 + tm.tm_mon, tm.tm_mday);
+                           latstr,
+                           lonstr,
+                           1900 + tm.tm_year,
+                           1 + tm.tm_mon,
+                           tm.tm_mday);
     g_debug ("iwin_start_open, requesting: %s", url);
     msg = soup_message_new ("GET", url);
     _gweather_info_begin_request (info, msg);
-    soup_session_queue_message (priv->session, msg, iwin_finish, info);
+    _gweather_info_queue_request (info, msg, iwin_finish);
 
     g_free (url);
 
