@@ -1,32 +1,20 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* weather-metar.c - Weather server functions (METAR)
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, see
- * <http://www.gnu.org/licenses/>.
+ * SPDX-FileCopyrightText: The GWeather authors
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
 
+#include "gweather-private.h"
+
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 
-#include "gweather-private.h"
-
-enum {
+enum
+{
     TIME_RE,
     WIND_RE,
     VIS_RE,
@@ -58,8 +46,8 @@ make_time (gint utcDate, gint utcHour, gint utcMin)
         tm.tm_mday = utcDate;
     }
     tm.tm_hour = utcHour;
-    tm.tm_min  = utcMin;
-    tm.tm_sec  = 0;
+    tm.tm_min = utcMin;
+    tm.tm_sec = 0;
 
     /* mktime() assumes value is local, not UTC.  Use tm_gmtoff to compensate */
 #ifdef HAVE_TM_TM_GMOFF
@@ -77,13 +65,12 @@ metar_tok_time (gchar *tokp, GWeatherInfo *info)
     g_debug ("metar_tok_time: %s", tokp);
 
     sscanf (tokp, "%2u%2u%2u", &day, &hr, &min);
-    info->priv->update = make_time (day, hr, min);
+    info->update = make_time (day, hr, min);
 }
 
 static void
 metar_tok_wind (gchar *tokp, GWeatherInfo *info)
 {
-    GWeatherInfoPrivate *priv;
     gchar sdir[4], sspd[4], sgust[4];
     gint dir, spd = -1;
     gchar *gustp;
@@ -91,8 +78,6 @@ metar_tok_wind (gchar *tokp, GWeatherInfo *info)
 
     g_debug ("metar_tok_wind: %s", tokp);
 
-    priv = info->priv;
-
     strncpy (sdir, tokp, 3);
     sdir[3] = 0;
     dir = (!strcmp (sdir, "VRB")) ? -1 : atoi (sdir);
@@ -106,112 +91,106 @@ metar_tok_wind (gchar *tokp, GWeatherInfo *info)
     gustp = strchr (tokp, 'G');
     if (gustp) {
         memset (sgust, 0, sizeof (sgust));
-	glen = strspn (gustp + 1, CONST_DIGITS);
+        glen = strspn (gustp + 1, CONST_DIGITS);
         strncpy (sgust, gustp + 1, glen);
-	tokp = gustp + 1 + glen;
+        tokp = gustp + 1 + glen;
     }
 
     if (!strcmp (tokp, "MPS"))
-	priv->windspeed = WINDSPEED_MS_TO_KNOTS ((GWeatherWindSpeed)spd);
+        info->windspeed = WINDSPEED_MS_TO_KNOTS ((GWeatherWindSpeed) spd);
     else
-	priv->windspeed = (GWeatherWindSpeed)spd;
+        info->windspeed = (GWeatherWindSpeed) spd;
 
     if ((349 <= dir) || (dir <= 11))
-        priv->wind = GWEATHER_WIND_N;
+        info->wind = GWEATHER_WIND_N;
     else if ((12 <= dir) && (dir <= 33))
-        priv->wind = GWEATHER_WIND_NNE;
+        info->wind = GWEATHER_WIND_NNE;
     else if ((34 <= dir) && (dir <= 56))
-        priv->wind = GWEATHER_WIND_NE;
+        info->wind = GWEATHER_WIND_NE;
     else if ((57 <= dir) && (dir <= 78))
-        priv->wind = GWEATHER_WIND_ENE;
+        info->wind = GWEATHER_WIND_ENE;
     else if ((79 <= dir) && (dir <= 101))
-        priv->wind = GWEATHER_WIND_E;
+        info->wind = GWEATHER_WIND_E;
     else if ((102 <= dir) && (dir <= 123))
-        priv->wind = GWEATHER_WIND_ESE;
+        info->wind = GWEATHER_WIND_ESE;
     else if ((124 <= dir) && (dir <= 146))
-        priv->wind = GWEATHER_WIND_SE;
+        info->wind = GWEATHER_WIND_SE;
     else if ((147 <= dir) && (dir <= 168))
-        priv->wind = GWEATHER_WIND_SSE;
+        info->wind = GWEATHER_WIND_SSE;
     else if ((169 <= dir) && (dir <= 191))
-        priv->wind = GWEATHER_WIND_S;
+        info->wind = GWEATHER_WIND_S;
     else if ((192 <= dir) && (dir <= 213))
-        priv->wind = GWEATHER_WIND_SSW;
+        info->wind = GWEATHER_WIND_SSW;
     else if ((214 <= dir) && (dir <= 236))
-        priv->wind = GWEATHER_WIND_SW;
+        info->wind = GWEATHER_WIND_SW;
     else if ((237 <= dir) && (dir <= 258))
-        priv->wind = GWEATHER_WIND_WSW;
+        info->wind = GWEATHER_WIND_WSW;
     else if ((259 <= dir) && (dir <= 281))
-        priv->wind = GWEATHER_WIND_W;
+        info->wind = GWEATHER_WIND_W;
     else if ((282 <= dir) && (dir <= 303))
-        priv->wind = GWEATHER_WIND_WNW;
+        info->wind = GWEATHER_WIND_WNW;
     else if ((304 <= dir) && (dir <= 326))
-        priv->wind = GWEATHER_WIND_NW;
+        info->wind = GWEATHER_WIND_NW;
     else if ((327 <= dir) && (dir <= 348))
-        priv->wind = GWEATHER_WIND_NNW;
+        info->wind = GWEATHER_WIND_NNW;
 }
 
 static void
 metar_tok_vis (gchar *tokp, GWeatherInfo *info)
 {
-    GWeatherInfoPrivate *priv;
     gchar *pfrac, *pend, *psp;
     gchar sval[6];
     gint num, den, val;
 
     g_debug ("metar_tok_vis: %s", tokp);
 
-    priv = info->priv;
-
     memset (sval, 0, sizeof (sval));
 
-    if (!strcmp (tokp,"CAVOK")) {
+    if (!strcmp (tokp, "CAVOK")) {
         // "Ceiling And Visibility OK": visibility >= 10 KM
-        priv->visibility=10000. / VISIBILITY_SM_TO_M (1.);
-        priv->sky = GWEATHER_SKY_CLEAR;
+        info->visibility = 10000. / VISIBILITY_SM_TO_M (1.);
+        info->sky = GWEATHER_SKY_CLEAR;
     } else if (0 != (pend = strstr (tokp, "SM"))) {
         // US observation: field ends with "SM"
         pfrac = strchr (tokp, '/');
         if (pfrac) {
-	    if (*tokp == 'M') {
-	        priv->visibility = 0.001;
-	    } else {
-	        num = (*(pfrac - 1) - '0');
-		strncpy (sval, pfrac + 1, pend - pfrac - 1);
-		den = atoi (sval);
-		priv->visibility =
-		    ((GWeatherVisibility)num / ((GWeatherVisibility)den));
+            if (*tokp == 'M') {
+                info->visibility = 0.001;
+            } else {
+                num = (*(pfrac - 1) - '0');
+                strncpy (sval, pfrac + 1, pend - pfrac - 1);
+                den = atoi (sval);
+                info->visibility =
+                    ((GWeatherVisibility) num / ((GWeatherVisibility) den));
 
-		psp = strchr (tokp, ' ');
-		if (psp) {
-		    *psp = '\0';
-		    val = atoi (tokp);
-		    priv->visibility += (GWeatherVisibility)val;
-		}
-	    }
-	} else {
-	    strncpy (sval, tokp, pend - tokp);
+                psp = strchr (tokp, ' ');
+                if (psp) {
+                    *psp = '\0';
+                    val = atoi (tokp);
+                    info->visibility += (GWeatherVisibility) val;
+                }
+            }
+        } else {
+            strncpy (sval, tokp, pend - tokp);
             val = atoi (sval);
-            priv->visibility = (GWeatherVisibility)val;
-	}
+            info->visibility = (GWeatherVisibility) val;
+        }
     } else {
         // International observation: NNNN(DD NNNNDD)?
         // For now: use only the minimum visibility and ignore its direction
         strncpy (sval, tokp, strspn (tokp, CONST_DIGITS));
-	val = atoi (sval);
-	priv->visibility = (GWeatherVisibility)val / VISIBILITY_SM_TO_M (1.);
+        val = atoi (sval);
+        info->visibility = (GWeatherVisibility) val / VISIBILITY_SM_TO_M (1.);
     }
 }
 
 static void
 metar_tok_cloud (gchar *tokp, GWeatherInfo *info)
 {
-    GWeatherInfoPrivate *priv;
     gchar stype[4], salt[4];
 
     g_debug ("metar_tok_cloud: %s", tokp);
 
-    priv = info->priv;
-
     strncpy (stype, tokp, 3);
     stype[3] = 0;
     if (strlen (tokp) == 6) {
@@ -220,27 +199,25 @@ metar_tok_cloud (gchar *tokp, GWeatherInfo *info)
     }
 
     if (!strcmp (stype, "CLR")) {
-        priv->sky = GWEATHER_SKY_CLEAR;
+        info->sky = GWEATHER_SKY_CLEAR;
     } else if (!strcmp (stype, "SKC")) {
-        priv->sky = GWEATHER_SKY_CLEAR;
+        info->sky = GWEATHER_SKY_CLEAR;
     } else if (!strcmp (stype, "NSC")) {
-        priv->sky = GWEATHER_SKY_CLEAR;
+        info->sky = GWEATHER_SKY_CLEAR;
     } else if (!strcmp (stype, "BKN")) {
-        priv->sky = GWEATHER_SKY_BROKEN;
+        info->sky = GWEATHER_SKY_BROKEN;
     } else if (!strcmp (stype, "SCT")) {
-        priv->sky = GWEATHER_SKY_SCATTERED;
+        info->sky = GWEATHER_SKY_SCATTERED;
     } else if (!strcmp (stype, "FEW")) {
-        priv->sky = GWEATHER_SKY_FEW;
+        info->sky = GWEATHER_SKY_FEW;
     } else if (!strcmp (stype, "OVC")) {
-        priv->sky = GWEATHER_SKY_OVERCAST;
+        info->sky = GWEATHER_SKY_OVERCAST;
     }
 }
 
 static void
 metar_tok_pres (gchar *tokp, GWeatherInfo *info)
 {
-    GWeatherInfoPrivate *priv = info->priv;
-
     g_debug ("metar_tok_pres: %s", tokp);
 
     if (*tokp == 'A') {
@@ -255,8 +232,8 @@ metar_tok_pres (gchar *tokp, GWeatherInfo *info)
         sfract[2] = 0;
         fract = atoi (sfract);
 
-        priv->pressure = (GWeatherPressure)intg + (((GWeatherPressure)fract)/100.0);
-    } else {  /* *tokp == 'Q' */
+        info->pressure = (GWeatherPressure) intg + (((GWeatherPressure) fract) / 100.0);
+    } else { /* *tokp == 'Q' */
         gchar spres[5];
         gint pres;
 
@@ -264,40 +241,37 @@ metar_tok_pres (gchar *tokp, GWeatherInfo *info)
         spres[4] = 0;
         pres = atoi (spres);
 
-        priv->pressure = PRESSURE_MBAR_TO_INCH ((GWeatherPressure)pres);
+        info->pressure = PRESSURE_MBAR_TO_INCH ((GWeatherPressure) pres);
     }
 }
 
 static void
 metar_tok_temp (gchar *tokp, GWeatherInfo *info)
 {
-    GWeatherInfoPrivate *priv;
     gchar *ptemp, *pdew, *psep;
 
     g_debug ("metar_tok_temp: %s", tokp);
 
-    priv = info->priv;
-
     psep = strchr (tokp, '/');
     *psep = 0;
     ptemp = tokp;
     pdew = psep + 1;
 
-    priv->temp = (*ptemp == 'M') ? TEMP_C_TO_F (-atoi (ptemp + 1))
-	: TEMP_C_TO_F (atoi (ptemp));
+    info->temp = (*ptemp == 'M') ? TEMP_C_TO_F (-atoi (ptemp + 1))
+                                 : TEMP_C_TO_F (atoi (ptemp));
     if (*pdew) {
-	priv->dew = (*pdew == 'M') ? TEMP_C_TO_F (-atoi (pdew + 1))
-	    : TEMP_C_TO_F (atoi (pdew));
+        info->dew = (*pdew == 'M') ? TEMP_C_TO_F (-atoi (pdew + 1))
+                                   : TEMP_C_TO_F (atoi (pdew));
     } else {
-	priv->dew = -1000.0;
+        info->dew = -1000.0;
     }
 }
 
 /* How "important" are the conditions to be reported to the user.
    Indexed by GWeatherConditionPhenomenon */
 static const int importance_scale[] = {
-    0, /* invalid */
-    0, /* none */
+    0,  /* invalid */
+    0,  /* none */
     20, /* drizzle */
     30, /* rain */
     35, /* snow */
@@ -325,16 +299,16 @@ static const int importance_scale[] = {
 
 static gboolean
 condition_more_important (GWeatherConditions *which,
-			  GWeatherConditions *than)
+                          GWeatherConditions *than)
 {
     if (!than->significant)
-	return TRUE;
+        return TRUE;
     if (!which->significant)
-	return FALSE;
+        return FALSE;
 
     if (importance_scale[than->phenomenon] <
-	importance_scale[which->phenomenon])
-	return TRUE;
+        importance_scale[which->phenomenon])
+        return TRUE;
 
     return FALSE;
 }
@@ -342,17 +316,14 @@ condition_more_important (GWeatherConditions *which,
 static void
 metar_tok_cond (gchar *tokp, GWeatherInfo *info)
 {
-    GWeatherInfoPrivate *priv;
     GWeatherConditions new_cond;
     gchar squal[3], sphen[4];
     gchar *pphen;
 
     g_debug ("metar_tok_cond: %s", tokp);
 
-    priv = info->priv;
-
     if ((strlen (tokp) > 3) && ((*tokp == '+') || (*tokp == '-')))
-        ++tokp;   /* FIX */
+        ++tokp; /* FIX */
 
     if ((*tokp == '+') || (*tokp == '-'))
         pphen = tokp + 1;
@@ -367,7 +338,7 @@ metar_tok_cond (gchar *tokp, GWeatherInfo *info)
 
     memset (sphen, 0, sizeof (sphen));
     strncpy (sphen, pphen, sizeof (sphen));
-    sphen[sizeof (sphen)-1] = '\0';
+    sphen[sizeof (sphen) - 1] = '\0';
 
     /* Defaults */
     new_cond.qualifier = GWEATHER_QUALIFIER_NONE;
@@ -455,15 +426,15 @@ metar_tok_cond (gchar *tokp, GWeatherInfo *info)
     if ((new_cond.qualifier != GWEATHER_QUALIFIER_NONE) || (new_cond.phenomenon != GWEATHER_PHENOMENON_NONE))
         new_cond.significant = TRUE;
 
-    if (condition_more_important (&new_cond, &priv->cond))
-	priv->cond = new_cond;
+    if (condition_more_important (&new_cond, &info->cond))
+        info->cond = new_cond;
 }
 
-#define TIME_RE_STR  "([0-9]{6})Z"
-#define WIND_RE_STR  "(([0-9]{3})|VRB)([0-9]?[0-9]{2})(G[0-9]?[0-9]{2})?(KT|MPS)"
-#define VIS_RE_STR   "((([0-9]?[0-9])|(M?([12] )?([1357]/1?[0-9])))SM)|" \
-    "([0-9]{4}(N|NE|E|SE|S|SW|W|NW( [0-9]{4}(N|NE|E|SE|S|SW|W|NW))?)?)|" \
-    "CAVOK"
+#define TIME_RE_STR "([0-9]{6})Z"
+#define WIND_RE_STR "(([0-9]{3})|VRB)([0-9]?[0-9]{2})(G[0-9]?[0-9]{2})?(KT|MPS)"
+#define VIS_RE_STR  "((([0-9]?[0-9])|(M?([12] )?([1357]/1?[0-9])))SM)|"                 \
+                   "([0-9]{4}(N|NE|E|SE|S|SW|W|NW( [0-9]{4}(N|NE|E|SE|S|SW|W|NW))?)?)|" \
+                   "CAVOK"
 #define COND_RE_STR  "(-|\\+)?(VC|MI|BC|PR|TS|BL|SH|DR|FZ)?(DZ|RA|SN|SG|IC|PE|GR|GS|UP|BR|FG|FU|VA|SA|HZ|PY|DU|SQ|SS|DS|PO|\\+?FC)"
 #define CLOUD_RE_STR "((CLR|BKN|SCT|FEW|OVC|SKC|NSC)([0-9]{3}|///)?(CB|TCU|///)?)"
 #define TEMP_RE_STR  "(M?[0-9][0-9])/(M?(//|[0-9][0-9])?)"
@@ -487,13 +458,18 @@ metar_init_re (void)
         return;
     initialized = TRUE;
 
-    metar_re[TIME_RE] = g_regex_new (RE_PREFIX TIME_RE_STR RE_SUFFIX, G_REGEX_OPTIMIZE, 0, NULL);
-    metar_re[WIND_RE] = g_regex_new (RE_PREFIX WIND_RE_STR RE_SUFFIX, G_REGEX_OPTIMIZE, 0, NULL);
-    metar_re[VIS_RE] = g_regex_new (RE_PREFIX VIS_RE_STR RE_SUFFIX, G_REGEX_OPTIMIZE, 0, NULL);
-    metar_re[COND_RE] = g_regex_new (RE_PREFIX COND_RE_STR RE_SUFFIX, G_REGEX_OPTIMIZE, 0, NULL);
-    metar_re[CLOUD_RE] = g_regex_new (RE_PREFIX CLOUD_RE_STR RE_SUFFIX, G_REGEX_OPTIMIZE, 0, NULL);
-    metar_re[TEMP_RE] = g_regex_new (RE_PREFIX TEMP_RE_STR RE_SUFFIX, G_REGEX_OPTIMIZE, 0, NULL);
-    metar_re[PRES_RE] = g_regex_new (RE_PREFIX PRES_RE_STR RE_SUFFIX, G_REGEX_OPTIMIZE, 0, NULL);
+    int re_flags = 0;
+#if !GLIB_CHECK_VERSION(2, 73, 2)
+    re_flags |= G_REGEX_OPTIMIZE;
+#endif
+
+    metar_re[TIME_RE] = g_regex_new (RE_PREFIX TIME_RE_STR RE_SUFFIX, re_flags, 0, NULL);
+    metar_re[WIND_RE] = g_regex_new (RE_PREFIX WIND_RE_STR RE_SUFFIX, re_flags, 0, NULL);
+    metar_re[VIS_RE] = g_regex_new (RE_PREFIX VIS_RE_STR RE_SUFFIX, re_flags, 0, NULL);
+    metar_re[COND_RE] = g_regex_new (RE_PREFIX COND_RE_STR RE_SUFFIX, re_flags, 0, NULL);
+    metar_re[CLOUD_RE] = g_regex_new (RE_PREFIX CLOUD_RE_STR RE_SUFFIX, re_flags, 0, NULL);
+    metar_re[TEMP_RE] = g_regex_new (RE_PREFIX TEMP_RE_STR RE_SUFFIX, re_flags, 0, NULL);
+    metar_re[PRES_RE] = g_regex_new (RE_PREFIX PRES_RE_STR RE_SUFFIX, re_flags, 0, NULL);
 
     metar_f[TIME_RE] = metar_tok_time;
     metar_f[WIND_RE] = metar_tok_wind;
@@ -509,7 +485,7 @@ metar_parse (gchar *metar, GWeatherInfo *info)
 {
     gchar *p;
     //gchar *rmk;
-    gint i, i2;
+    gint i2;
     gchar *tokp;
 
     g_return_val_if_fail (info != NULL, FALSE);
@@ -530,26 +506,26 @@ metar_parse (gchar *metar, GWeatherInfo *info)
     }
 
     p = metar;
-    i = TIME_RE;
     while (*p) {
         int token_start, token_end;
 
         i2 = RE_NUM;
-        token_start = strlen(p);
+        token_start = strlen (p);
         token_end = token_start;
 
-        for (i = 0; i < RE_NUM; i++) {
+        for (int i = 0; i < RE_NUM; i++) {
             GMatchInfo *match_info;
 
-            if (g_regex_match_full (metar_re[i], p, -1, 0, 0, &match_info, NULL))
-            {
+            if (g_regex_match_full (metar_re[i], p, -1, 0, 0, &match_info, NULL)) {
                 int tmp_token_start, tmp_token_end;
                 /* Skip leading and trailing space characters, if present.
                    (the regular expressions include those characters to
                    only get matches limited to whole words). */
                 g_match_info_fetch_pos (match_info, 0, &tmp_token_start, &tmp_token_end);
-                if (p[tmp_token_start] == ' ') tmp_token_start++;
-                if (p[tmp_token_end - 1] == ' ') tmp_token_end--;
+                if (p[tmp_token_start] == ' ')
+                    tmp_token_start++;
+                if (p[tmp_token_end - 1] == ' ')
+                    tmp_token_end--;
 
                 /* choose the regular expression with the earliest match */
                 if (tmp_token_start < token_start) {
@@ -564,7 +540,7 @@ metar_parse (gchar *metar, GWeatherInfo *info)
 
         if (i2 != RE_NUM) {
             tokp = g_strndup (p + token_start, token_end - token_start);
-            metar_f[i2] (tokp, info);
+            metar_f[i2](tokp, info);
             g_free (tokp);
         }
 
@@ -575,66 +551,79 @@ metar_parse (gchar *metar, GWeatherInfo *info)
 }
 
 static void
-metar_finish (SoupSession *session, SoupMessage *msg, gpointer data)
+metar_finish (GObject *source, GAsyncResult *result, gpointer data)
 {
+    SoupSession *session = SOUP_SESSION (source);
+    SoupMessage *msg = soup_session_get_async_result_message (session, result);
+    g_autoptr (GError) error = NULL;
+    GBytes *body;
     GWeatherInfo *info;
-    GWeatherInfoPrivate *priv;
     WeatherLocation *loc;
-    const gchar *p, *eoln;
+    const gchar *p, *eoln, *response_body;
     gchar *searchkey, *metar;
     gboolean success = FALSE;
 
-    if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
-        if (msg->status_code == SOUP_STATUS_CANCELLED) {
-	    g_debug ("Failed to get METAR data: %d %s.\n",
-		     msg->status_code, msg->reason_phrase);
-	    return;
-	}
-
-	info = data;
-	if (SOUP_STATUS_IS_TRANSPORT_ERROR (msg->status_code)) {
-	    info->priv->network_error = TRUE;
-	} else {
-	    /* Translators: %d is an error code, and %s the error string */
-	    g_warning (_("Failed to get METAR data: %d %s.\n"),
-		       msg->status_code, msg->reason_phrase);
-	}
-
-	_gweather_info_request_done (info, msg);
-	return;
-    }
-
     info = data;
-    priv = info->priv;
-    loc = &priv->location;
+    loc = &info->location;
+
+    body = soup_session_send_and_read_finish (session, result, &error);
+    if (!body) {
+        if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+            g_debug ("Failed to get METAR data: %s", error->message);
+            return;
+        }
+        g_warning ("Failed to get METAR data: %s", error->message);
+        _gweather_info_request_done (info, msg);
+        return;
+    } else if (!SOUP_STATUS_IS_SUCCESSFUL (soup_message_get_status (msg))) {
+        g_bytes_unref (body);
+        info->network_error = TRUE;
+        _gweather_info_request_done (info, msg);
+        return;
+    } else if (soup_message_get_status (msg) == 204) {
+        g_debug ("Data not available for METAR %s", loc->code);
+        _gweather_info_request_done (info, msg);
+        g_bytes_unref (body);
+        return;
+    }
+    response_body = g_bytes_get_data (body, NULL);
 
     g_debug ("METAR data for %s", loc->code);
-    g_debug ("%s", msg->response_body->data);
+    g_debug ("%s", response_body);
 
-    searchkey = g_strdup_printf ("<raw_text>%s ", loc->code);
-    p = strstr (msg->response_body->data, searchkey);
+    searchkey = g_strdup_printf ("<raw_text>METAR %s ", loc->code);
+    p = strstr (response_body, searchkey);
 
     if (p) {
-	p += strlen (searchkey);
-	eoln = strstr (p, "</raw_text>");
-	if (eoln)
-	    metar = g_strndup (p, eoln - p);
-	else
-	    metar = g_strdup (p);
-	success = metar_parse (metar, info);
-	g_free (metar);
-    } else if (!strstr (msg->response_body->data, "aviationweather.gov")) {
-	/* The response doesn't even seem to have come from NOAA...
+        p += strlen (searchkey);
+        eoln = strstr (p, "</raw_text>");
+        if (eoln)
+            metar = g_strndup (p, eoln - p);
+        else
+            metar = g_strdup (p);
+        success = metar_parse (metar, info);
+        g_free (metar);
+        if (success)
+            g_debug ("Successfully parsed METAR for %s", loc->code);
+        else
+            g_debug ("Failed to parse raw_text METAR for %s", loc->code);
+    } else if (!strstr (response_body, "aviationweather.gov")) {
+        /* The response doesn't even seem to have come from NOAA...
 	 * most likely it is a wifi hotspot login page. Call that a
 	 * network error.
 	 */
-	priv->network_error = TRUE;
+        info->network_error = TRUE;
+        g_debug ("Response to query for %s did not come from correct server", loc->code);
+    } else {
+        g_debug ("Failed to parse METAR for %s", loc->code);
     }
 
     g_free (searchkey);
 
-    if (!priv->valid)
-        priv->valid = success;
+    if (!info->valid)
+        info->valid = success;
+
+    g_bytes_unref (body);
     _gweather_info_request_done (info, msg);
 }
 
@@ -642,31 +631,51 @@ metar_finish (SoupSession *session, SoupMessage *msg, gpointer data)
 void
 metar_start_open (GWeatherInfo *info)
 {
-    GWeatherInfoPrivate *priv;
     WeatherLocation *loc;
     SoupMessage *msg;
+    GUri *uri;
+    char *query;
 
     g_return_if_fail (info != NULL);
 
-    priv = info->priv;
-
-    priv->valid = priv->network_error = FALSE;
-    loc = &priv->location;
+    info->valid = info->network_error = FALSE;
+    loc = &info->location;
 
     if (!loc->latlon_valid)
         return;
 
-    g_debug ("metar_start_open, requesting: https://www.aviationweather.gov/adds/dataserver_current/httpparam?dataSource=metars&requestType=retrieve&format=xml&hoursBeforeNow=3&mostRecent=true&fields=raw_text&stationString=%s", loc->code);
-    msg = soup_form_request_new (
-	"GET", "https://www.aviationweather.gov/adds/dataserver_current/httpparam",
-	"dataSource", "metars",
-	"requestType", "retrieve",
-	"format", "xml",
-	"hoursBeforeNow", "3",
-	"mostRecent", "true",
-	"fields", "raw_text",
-	"stationString", loc->code,
-	NULL);
+    g_debug ("metar_start_open, requesting: https://aviationweather.gov/api/data/dataserver?dataSource=metars&requestType=retrieve&format=xml&hoursBeforeNow=3&mostRecent=true&fields=raw_text&stationString=%s", loc->code);
+    query = soup_form_encode (
+        "dataSource",
+        "metars",
+        "requestType",
+        "retrieve",
+        "format",
+        "xml",
+        "hoursBeforeNow",
+        "3",
+        "mostRecent",
+        "true",
+        "fields",
+        "raw_text",
+        "stationString",
+        loc->code,
+        NULL);
+
+    uri = g_uri_build (SOUP_HTTP_URI_FLAGS,
+                       "https",
+                       NULL,
+                       "aviationweather.gov",
+                       -1,
+                       "/api/data/dataserver",
+                       query,
+                       NULL);
+    g_free (query);
+
+    msg = soup_message_new_from_uri ("GET", uri);
     _gweather_info_begin_request (info, msg);
-    soup_session_queue_message (priv->session, msg, metar_finish, info);
+    _gweather_info_queue_request (info, msg, metar_finish);
+    g_object_unref (msg);
+
+    g_uri_unref (uri);
 }
